defmodule DrainCloudCoreWeb.AuthController do
  use DrainCloudCoreWeb, :controller

  alias DrainCloudCoreWeb.AuthController.RegisterRequest, as: Request
  alias :logger, as: Log

  alias DrainCloudCore.Repo, as: Repo

  def register(conn = %Plug.Conn{}, _params) do
    req = Request.from_request(conn)
    Log.debug("new register request: #{req}")

    model = Request.to_model(req)

    try do
      Repo.insert(model)
    rescue
      e in RuntimeError -> Log.error("failed to insert new user: #{e}")
    end

    json(conn, req)
  end
end

defmodule DrainCloudCoreWeb.AuthController.RegisterRequest do
  @derive [Poison.Encoder, Jason.Encoder]

  defstruct login: "", password: ""

  alias __MODULE__, as: Request
  alias DrainCloudCore.Auth.Users, as: User
  alias DrainCloudCoreWeb.Errors.InvalidArgumentException, as: InvalidArgumentException

  def from_request(conn) do
    res = Plug.Conn.read_body(conn)
    :logger.warning(Kernel.inspect(res))

    case res do
      {:error, _} ->
        raise "failed to read request body: #{:error}"

      {_, data, _} ->
        req = Poison.decode!(data, as: %Request{})
        validate_request(req)
        req

      _ ->
        raise "failed to read request body. unexpected result #{Kernel.inspect(res)}"
    end
  end

  def to_model(req = %Request{}) do
    %User{
      login: req.login,
      password: req.password
    }
  end

  defp validate_request(req = %Request{}) do
    if String.length(req.login) < 3 do
      raise InvalidArgumentException, %{
        message: "login must be 3 symbols at minimum",
        args: ["login"]
      }
    else
      if String.length(req.password) < 8 do
        raise InvalidArgumentException, %{
          message: "password must be 8 symbols at minimum",
          args: ["password"]
        }
      end
    end
  end

  defimpl String.Chars, for: DrainCloudCoreWeb.AuthController.RegisterRequest do
    def to_string(req) do
      case Jason.encode(req) do
        {:ok, str} -> str
        {:error, msg} -> raise "failed to encode register request to string: #{msg}"
      end
    end
  end
end
